@@ -1,4 +1,4 @@
-# Copyright (C) 2011 by Brandon Invergo (b.invergo@gmail.com)
+# Copyright (C) 2011, 2016 by Brandon Invergo (b.invergo@gmail.com)
 # This code is part of the Biopython distribution and governed by its
 # license. Please see the LICENSE file that should have been included
 # as part of this package.
@@ -80,15 +80,13 @@ def parse_basics(lines, results):
             else:
                 multi_models = True
         # Get the maximum log-likelihood
-        if "ln Lmax" in line and len(line_floats) > 0:
+        if "ln Lmax" in line and line_floats:
             results["lnL max"] = line_floats[0]
     return (results, multi_models, multi_genes)
 
 
 def parse_nssites(lines, results, multi_models, multi_genes):
-    """Determine which NSsites models are present and parse them.
-    """
-
+    """Determine which NSsites models are present and parse them."""
     ns_sites = {}
     model_re = re.compile("Model (\d+):\s+(.+)")
     gene_re = re.compile("Gene\s+([0-9]+)\s+.+")
@@ -99,11 +97,12 @@ def parse_nssites(lines, results, multi_models, multi_genes):
         if siteclass_model is None:
             siteclass_model = "one-ratio"
         current_model = {"one-ratio": 0,
-                        "NearlyNeutral": 1,
-                        "PositiveSelection": 2,
-                        "discrete": 3,
-                        "beta": 7,
-                        "beta&w>1": 8}[siteclass_model]
+                         "NearlyNeutral": 1,
+                         "PositiveSelection": 2,
+                         "discrete": 3,
+                         "beta": 7,
+                         "beta&w>1": 8,
+                         "M2a_rel": 22}[siteclass_model]
         if multi_genes:
             genes = results["genes"]
             current_gene = None
@@ -185,7 +184,7 @@ def parse_model(lines, results):
         # Find lnL values.
         # Example match (lnL = -2021.348300):
         # "lnL(ntime: 19  np: 22):  -2021.348300      +0.000000"
-        if "lnL(ntime:" in line and len(line_floats) > 0:
+        if "lnL(ntime:" in line and line_floats:
             results["lnL"] = line_floats[0]
             np_res = re.match("lnL\(ntime:\s+\d+\s+np:\s+(\d+)\)", line)
             if np_res is not None:
@@ -199,7 +198,7 @@ def parse_model(lines, results):
         elif len(line_floats) == num_params and not SEs_flag:
             parameters["parameter list"] = line.strip()
         # Find SEs. The same format as parameters above is maintained
-        # since there is a correspondance between the SE format and
+        # since there is a correspondence between the SE format and
         # the parameter format.
         # Example match:
         # "SEs for parameters:
@@ -211,7 +210,7 @@ def parse_model(lines, results):
             SEs_flag = False
         # Find tree lengths.
         # Example match: "tree length =   1.71931"
-        elif "tree length =" in line and len(line_floats) > 0:
+        elif "tree length =" in line and line_floats:
             results["tree length"] = line_floats[0]
         # Find the estimated trees only taking the tree if it has
         # lengths or rate estimates on the branches
@@ -236,18 +235,18 @@ def parse_model(lines, results):
                 w_tree_flag = True
         # Find rates for multiple genes
         # Example match: "rates for 2 genes:     1  2.75551"
-        elif "rates for" in line and len(line_floats) > 0:
+        elif "rates for" in line and line_floats:
             line_floats.insert(0, 1.0)
             parameters["rates"] = line_floats
         # Find kappa values.
         # Example match: "kappa (ts/tv) =  2.77541"
-        elif "kappa (ts/tv)" in line and len(line_floats) > 0:
+        elif "kappa (ts/tv)" in line and line_floats:
             parameters["kappa"] = line_floats[0]
         # Find omega values.
         # Example match: "omega (dN/dS) =  0.25122"
-        elif "omega (dN/dS)" in line and len(line_floats) > 0:
+        elif "omega (dN/dS)" in line and line_floats:
             parameters["omega"] = line_floats[0]
-        elif "w (dN/dS)" in line and len(line_floats) > 0:
+        elif "w (dN/dS)" in line and line_floats:
             parameters["omega"] = line_floats
         # Find omega and kappa values for multi-gene files
         # Example match: "gene # 1: kappa =   1.72615 omega =   0.39333"
@@ -256,14 +255,14 @@ def parse_model(lines, results):
             if parameters.get("genes") is None:
                 parameters["genes"] = {}
             parameters["genes"][gene_num] = {"kappa": line_floats[0],
-                                            "omega": line_floats[1]}
+                                             "omega": line_floats[1]}
         # Find dN values.
         # Example match: "tree length for dN:       0.2990"
-        elif "tree length for dN" in line and len(line_floats) > 0:
+        elif "tree length for dN" in line and line_floats:
             parameters["dN"] = line_floats[0]
         # Find dS values
         # Example match: "tree length for dS:       1.1901"
-        elif "tree length for dS" in line and len(line_floats) > 0:
+        elif "tree length for dS" in line and line_floats:
             parameters["dS"] = line_floats[0]
         # Find site class distributions.
         # Example match 1 (normal model, 2 site classes):
@@ -289,7 +288,7 @@ def parse_model(lines, results):
                 site_classes = parameters.get("site classes")
                 branch_type_no = int(branch_type.group(1))
                 site_classes = parse_clademodelc(branch_type_no, line_floats,
-                        site_classes)
+                                                 site_classes)
                 parameters["site classes"] = site_classes
         # Find the omega values of the foreground branch for each site
         # class in the branch site A model
@@ -312,7 +311,7 @@ def parse_model(lines, results):
         # method.
         # Example row (some spaces removed to make it smaller...).
         # " 6..7   0.000  167.7  54.3  0.0000  0.0000  0.0000  0.0  0.0"
-        elif branch_res is not None and len(line_floats) > 0:
+        elif branch_res is not None and line_floats:
             branch = branch_res.group(1)
             if parameters.get("branches") is None:
                 parameters["branches"] = {}
@@ -333,28 +332,30 @@ def parse_model(lines, results):
         # Example matches:
         # "  p0=  0.99043  p=  0.36657 q=  1.04445
         # "  (p1=  0.00957) w=  3.25530"
-        elif len(model_params) > 0:
+        elif model_params:
             float_model_params = []
             for param in model_params:
                 float_model_params.append((param[0], _nan_float(param[1])))
             parameters.update(dict(float_model_params))
-    if len(parameters) > 0:
+    if parameters:
         results["parameters"] = parameters
     return results
 
 
 def parse_siteclass_proportions(line_floats):
-    """For models which have multiple site classes, find the proportion of the alignment assigned to each class.
+    """For models which have multiple site classes, find the proportion of the
+    alignment assigned to each class.
     """
     site_classes = {}
-    if len(line_floats) > 0:
+    if line_floats:
         for n in range(len(line_floats)):
             site_classes[n] = {"proportion": line_floats[n]}
     return site_classes
 
 
 def parse_siteclass_omegas(line, site_classes):
-    """For models which have multiple site classes, find the omega estimated for each class.
+    """For models which have multiple site classes, find the omega estimated
+    for each class.
     """
     # The omega results are tabular with strictly 9 characters per column
     # (1 to 3 digits before the  decimal point and 5 after). This causes
@@ -390,11 +391,9 @@ def parse_branch_site_a(foreground, line_floats, site_classes):
         if site_classes[n].get("branch types") is None:
             site_classes[n]["branch types"] = {}
         if foreground:
-            site_classes[n]["branch types"]["foreground"] =\
-                    line_floats[n]
+            site_classes[n]["branch types"]["foreground"] = line_floats[n]
         else:
-            site_classes[n]["branch types"]["background"] =\
-                    line_floats[n]
+            site_classes[n]["branch types"]["background"] = line_floats[n]
     return site_classes
 
 
@@ -410,6 +409,8 @@ def parse_pairwise(lines, results):
     # t= 0.0126  S=    81.4  N=   140.6  dN/dS= 0.0010  dN= 0.0000  dS= 0.0115
     pair_re = re.compile("\d+ \((.+)\) ... \d+ \((.+)\)")
     pairwise = {}
+    seq1 = None
+    seq2 = None
     for line in lines:
         # Find all floating point numbers in this line
         line_floats_res = line_floats_re.findall(line)
@@ -418,22 +419,22 @@ def parse_pairwise(lines, results):
         if pair_res:
             seq1 = pair_res.group(1)
             seq2 = pair_res.group(2)
-            if pairwise.get(seq1) is None:
+            if seq1 not in pairwise:
                 pairwise[seq1] = {}
-            if pairwise.get(seq2) is None:
+            if seq2 not in pairwise:
                 pairwise[seq2] = {}
-            if len(line_floats) == 1:
-                pairwise[seq1][seq2] = {"lnL": line_floats[0]}
-                pairwise[seq2][seq1] = pairwise[seq1][seq2]
-            elif len(line_floats) == 6:
-                pairwise[seq1][seq2] = {"t": line_floats[0],
-                        "S": line_floats[1],
-                        "N": line_floats[2],
-                        "omega": line_floats[3],
-                        "dN": line_floats[4],
-                        "dS": line_floats[5]}
-                pairwise[seq2][seq1] = pairwise[seq1][seq2]
-    if len(pairwise) > 0:
+        if len(line_floats) == 1 and seq1 is not None and seq2 is not None:
+            pairwise[seq1][seq2] = {"lnL": line_floats[0]}
+            pairwise[seq2][seq1] = pairwise[seq1][seq2]
+        elif len(line_floats) == 6 and seq1 is not None and seq2 is not None:
+            pairwise[seq1][seq2].update({"t": line_floats[0],
+                                         "S": line_floats[1],
+                                         "N": line_floats[2],
+                                         "omega": line_floats[3],
+                                         "dN": line_floats[4],
+                                         "dS": line_floats[5]})
+            pairwise[seq2][seq1] = pairwise[seq1][seq2]
+    if pairwise:
         results["pairwise"] = pairwise
     return results
 
@@ -461,7 +462,7 @@ def parse_distances(lines, results):
         # Parse AA distances (raw or ML), in a lower diagonal matrix
         matrix_row_res = matrix_row_re.match(line)
         if matrix_row_res and (raw_aa_distances_flag or
-                ml_aa_distances_flag):
+                               ml_aa_distances_flag):
             seq_name = matrix_row_res.group(1).strip()
             if seq_name not in sequences:
                 sequences.append(seq_name)
@@ -479,6 +480,6 @@ def parse_distances(lines, results):
                 for i in range(0, len(line_floats)):
                     distances["ml"][seq_name][sequences[i]] = line_floats[i]
                     distances["ml"][sequences[i]][seq_name] = line_floats[i]
-    if len(distances) > 0:
+    if distances:
         results["distances"] = distances
     return results
